;;; Programa: Interfaz para la creacin de sistemas de 
;;;           produccin simples
;;; Autor: Vctor Rodrguez Herreros
;;;        Escuela Tcnica Superior de Ingeniera
;;;        Informtica (www.etsii.ull.es)
;;; Fecha: 29 de Septiembre de 1997
;;; Comentarios:
;;;    La finalidad de este programa es la de crear una 
;;;    herramienta en Lisp que permita la creacin de 
;;;    sistemas de produccin para la resolucin de 
;;;    diferentes problemas. Por lo tanto, esta interfaz 
;;;    es independiente del tipo de problema y no hace 
;;;    ninguna suposicin respecto al tipo de problema
;;;
;;; Historia:
;;;   2 de Octubre de 1997 - Terminada la implementacion de 
;;;                          las funciones REGLA e HIJOS
;;;   Mayo 1998 - Modificaciones de maquillaje (Patricio)
;;;   Mayo 2004 - Cambio para generar paquete shell (Patricio)

;;; ****************************************************
;;; Documentacin **************************************
;;; ****************************************************
;;;
;;; SHELL - Interfaz para la implementacin de sistemas 
;;;         de produccin
;;; Programado originalmente con CLISP para OS/2
;;; Adaptado para su uso genrico en ANSI COMMON LISP
;;; 
;;; Uso de la libera:
;;;
;;; (LOAD "shell")
;;; (USE-PACKAGE "shell")
;;;
;;; Carga en memoria el paquete y lo hace accesible a
;;; ser usado.
;;; 
;;; (REGLA PRECONDICION ACCION)
;;; 
;;; Esta funcin se utiliza para introducir reglas en el 
;;; sistema de produccin. El primer parmetro es la funcin
;;; booleana que evalua una precondicin para un estado 
;;; dado. El segundo parmetro se corresponde con la funcin
;;; que crea el estado correspondiente a la accin aplicada 
;;; sobre un estado cuando en ste se cumpla la precondicin 
;;; evualuada por el primer parmetro de la funcin.
;;; Por tanto tanto las funciones PRECONDICION como ACCION 
;;; reciben como parmetros un estado y devuelven T o NIL y 
;;; el nuevo estado respectivamente.
;;; 
;;; (HIJOS PADRE)
;;; 
;;; Esta funcin se utiliza para obtener los nuevos estados 
;;; a los que se puede llegar mediante la aplicacin de las 
;;; reglas del sistema de produccin creado previamente 
;;; utilizando la funcin REGLA. Se retorna una lista con
;;; todos los estados a los que se ha podido evolucionar 
;;; desde el estado pasado como parmetro.
;;; 
;;; *SISTEMA*
;;; 
;;; Esta variable es de uso interno de la librera y no 
;;; debera ser modificada directamente por el usuario. 
;;; En ella se almacena el sistema de produccion creado 
;;; con la funcion REGLA.
;;; 

;;; Inicializamos paquete
;;; 

(make-package "shell" :nicknames '(sh) :use '("LISP"))
(in-package "shell")

(export '(*sistema* regla hijos))

;;; La siguiente variable almacena todas las reglas que 
;;; se han creado para el sistema de produccion. 
;;; Se inicializa con el valor lista vacia = NIL

(defvar *sistema* nil
  "Esta variable almacena en una lista todas las reglas que se han creado para el sistema de produccin.")

;;; La funcion REGLA se encarga de aadir una nueva regla 
;;; al sistema de produccin. Se pasan dos parmetros, la
;;; precondicin de la misma y la accin a ejecutar en 
;;; caso de que la precondicin se cumpla
(defun regla (precondicion accion)
  "
    - Funcion: REGLA (PRECONDICION ACCION)

        Funcion que se encarga de aadir una nueva regla al sistema de produccin (almacenado en la variable *sistema*). Se le pasan dos parmetros, la precondicin de la misma y la accin a ejecutar en caso de que la precondicin se cumpla"
  ; Se comprueba si es la primera regla a aadir
  (push (list precondicion 
	      accion )
	*sistema*))
; Final del DEFUN REGLA

;;; La funcion HIJOS devuelve una lista con todos los 
;;; estados a los que se puede evolucionar siempre y 
;;; cuando se cumpla la precondicin de la regla 
;;; correspondiente. El estado desde el que se genera
;;; esta lista de estados se pasar como parmetro a  
;;; funcin
(defun hijos (padre)
  "Funcin que devuelve una lista con todos los estados a los que se puede evolucionar siempre y cuando se cumpla la precondicin de la regla correspondiente. El estado desde el que se genera esta lista de estados se pasar como parmetro a la funcin"
  ; La variable local LISTA almacenara a los estados 
  ; generados por la aplicacion de las reglas de nuestro 
  ; sistema de produccion
  (let ((lista nil))
    ; Para cada regla del sistema de produccion ...
    (dolist (patron *sistema* lista)
      ; Si se cumple la precondicion ...
      (when (funcall (first patron) padre)
      ; Ejecutamos la precondicion. Hay que aadir el 
      ; nuevo estado a la lista de hijos generados
	(push (funcall (first (rest patron)) padre) lista)))))


